﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Serialization;

namespace Product_API_Availability_Request
{
    class Program
    {
        static void Main(string[] args)
        {
            TransferAvail();
        }

        static void TransferAvail()
        {
            // 1. Parameters (parameters you may specify on the request)

            string User = "InsertUserHere";
            string Pass = "InsertPassHere";

            string Language = "en";

            int AdultsNumber = 2;
            int ChildrenNumber = 0;
            string ChildrenAgesString = ""; // 2 digit children ages (Example: "0815", 2 children, first one of 8 years old and second one of 15 years old).

            DateTime StartDate = DateTime.Now.AddDays(30);
            DateTime EndDate = DateTime.Now.AddDays(31).AddHours(2);

            int OriginZone = 15011;
            int DestinationZone = 15011; // Destination code for Palma de Mallorca in the test environment

            bool InOut = false;

            // 2. Initialization (this code initializes all of the neccesaries nodes and propestier of the TransferAvail request)

            WebServiceJP.JP_TransferAvailabilityRQ AvailRQ = new WebServiceJP.JP_TransferAvailabilityRQ();
            AvailRQ.Version = "1.1";
            AvailRQ.Language = Language;

            // 2.1. Login

            WebServiceJP.JP_Login Login = new WebServiceJP.JP_Login();
            Login.Email = User;
            Login.Password = Pass;
            AvailRQ.Login = Login;

            // 2.2. Request

            WebServiceJP.JP_TransferRequest ProductRequest = new WebServiceJP.JP_TransferRequest();
            WebServiceJP.JP_SearchSegmentsTransfers SearchSegmentsTransfers = new WebServiceJP.JP_SearchSegmentsTransfers();
            WebServiceJP.JP_SearchSegmentTransfers SearchSegmenTransfers = new WebServiceJP.JP_SearchSegmentTransfers();

            SearchSegmenTransfers.Start = StartDate;
            SearchSegmenTransfers.End = EndDate;

            SearchSegmenTransfers.HourSpecified = true;
            SearchSegmenTransfers.Hour = StartDate;
            SearchSegmenTransfers.HourEndSpecified = true;
            SearchSegmenTransfers.HourEnd = EndDate;

            SearchSegmenTransfers.OriginZoneSpecified = true;
            SearchSegmenTransfers.OriginZone = OriginZone;
            SearchSegmenTransfers.DestinationZoneSpecified = true;
            SearchSegmenTransfers.DestinationZone = DestinationZone;

            if (InOut)
            {
                SearchSegmenTransfers.RouteSpecified = true;
                SearchSegmenTransfers.Route = WebServiceJP.JP_RouteType.InOut;
            }

            SearchSegmentsTransfers.SearchSegmenTransfers = SearchSegmenTransfers;
            ProductRequest.SearchSegmentsTransfers = SearchSegmentsTransfers;

            // 2.3. Passengers (Paxes/RelPaxesDist)

            // 2.3.1. Paxes: List of passengers and their ages
            WebServiceJP.JP_Paxes Paxes = new WebServiceJP.JP_Paxes();
            WebServiceJP.JP_Pax[] Pax;
            List<WebServiceJP.JP_Pax> PaxList = new List<WebServiceJP.JP_Pax>();

            // 2.3.2. RelPaxesDist: Distribution of passengers between rooms

            WebServiceJP.JP_RelPaxDist[] RelPaxDist;
            List<WebServiceJP.JP_RelPaxDist> RelPaxDistList = new List<WebServiceJP.JP_RelPaxDist>();
            WebServiceJP.JP_RelPaxDist RelPaxDist1 = new WebServiceJP.JP_RelPaxDist();
            WebServiceJP.JP_RelPax[] RelPax;
            List<WebServiceJP.JP_RelPax> RelPaxList = new List<WebServiceJP.JP_RelPax>();

            int passenger = 1;

            for (int adult = 1; adult <= AdultsNumber; adult += +1)
            {
                //Paxes (List of all available paxes)
                WebServiceJP.JP_Pax Pax1 = new WebServiceJP.JP_Pax();
                Pax1.IdPax = passenger;
                Pax1.AgeSpecified = true;
                Pax1.Age = 30;
                PaxList.Add(Pax1);

                //RelPaxesDist (Passengers of each room)
                WebServiceJP.JP_RelPax RelPax1 = new WebServiceJP.JP_RelPax();
                RelPax1.IdPax = passenger;
                RelPaxList.Add(RelPax1);

                passenger++;
            }

            for (int nino = 1; nino <= ChildrenNumber; nino += +1)
            {
                //Paxes (List of all available paxes)
                WebServiceJP.JP_Pax Pax1 = new WebServiceJP.JP_Pax();
                Pax1.IdPax = passenger;
                Pax1.AgeSpecified = true;

                if (ChildrenAgesString.Length < ((ChildrenNumber - 1) * 2) + 2)
                    throw new System.ArgumentException("ChildrenAgesString has not been properly specified.");

                Pax1.Age = Int32.Parse(ChildrenAgesString.Substring((ChildrenNumber - 1) * 2, 2));

                PaxList.Add(Pax1);

                //RelPaxesDist (Passengers of each room)
                WebServiceJP.JP_RelPax RelPax1 = new WebServiceJP.JP_RelPax();
                RelPax1.IdPax = passenger;
                RelPaxList.Add(RelPax1);

                passenger++;
            }

            //RelPaxesDist (Room distribution being added to the room distribution list)
            RelPax = RelPaxList.ToArray();
            RelPaxDist1.RelPaxes = RelPax;
            RelPaxDistList.Add(RelPaxDist1);
            RelPaxDist = RelPaxDistList.ToArray();
            ProductRequest.RelPaxesDist = RelPaxDist;

            //Paxes (List of all available paxes being added to the request)
            Pax = PaxList.ToArray();
            Paxes.Pax = Pax;
            AvailRQ.Paxes = Paxes;

            AvailRQ.TransferRequest = ProductRequest;

            // 2.3. AdvancedOptions (optional parameters)

            WebServiceJP.JP_ServicesAvailAdvancedOptions AdvancedOptions = new WebServiceJP.JP_ServicesAvailAdvancedOptions();
            AdvancedOptions.ShowBreakdownPriceSpecified = true;
            AdvancedOptions.ShowBreakdownPrice = true;

            AvailRQ.AdvancedOptions = AdvancedOptions;

            // 3. Avail Request (here we prepare and send the Avail request)

            WebServiceJP.AvailTransactionsClient serv = new WebServiceJP.AvailTransactionsClient();

            Console.WriteLine(string.Format("- AVAIL REQUEST"));
            Console.WriteLine(string.Empty);
            Console.WriteLine(string.Format("Endpoint: {0}", serv.Endpoint.Address));
            Console.WriteLine(string.Format("User: {0}", User));
            Console.WriteLine(string.Format("Pass: {0}", Pass));
            Console.WriteLine(string.Format("Dates: {0} - {1}", StartDate.ToShortDateString(), EndDate.ToShortDateString()));
            Console.WriteLine(string.Format("OriginCode: {0}", OriginZone));
            Console.WriteLine(string.Format("DestinationCode: {0}", DestinationZone));
            Console.WriteLine(string.Empty);

            WebServiceJP.JP_AvailResponseRS RST = serv.TransferAvail(AvailRQ);

            // 4. Avail Response (here we process the Avail response)

            Console.WriteLine(string.Format("- AVAIL RESPONSE"));
            Console.WriteLine(string.Empty);
            if (RST != null && RST.Results != null)
            {
                int OptionsCount = 0;
                int Count = 0;
                foreach (WebServiceJP.JP_TransferResult item in RST.Results.Items)
                {

                    Count += 1;
                    foreach (WebServiceJP.JP_TransferOption transferOption in item.TransferOptions)
                        OptionsCount += 1;
                }

                Console.WriteLine(string.Format("OK"));
                Console.WriteLine(string.Format("Product: {0}", Count));
                Console.WriteLine(string.Format("ProductOptions: {0}", OptionsCount));
            }
            else if (RST != null && RST.Errors != null)
            {
                Console.WriteLine(string.Format("KO"));
                Console.WriteLine(string.Format("Error: {0}", RST.Errors[0].Code));
            }
            else
            {
                Console.WriteLine(string.Format("KO"));
                Console.WriteLine(string.Format("Error: UNKNOWN ERROR"));
            }

            // 5. XML serialization examples (for when you require for the XML logs any of the requests/reponses)

            string rqXML = serializeXML<WebServiceJP.JP_TransferAvailabilityRQ>(AvailRQ);
            string rsXML = serializeXML<WebServiceJP.JP_AvailResponseRS>(RST);

            Console.ReadLine();
        }

        static string serializeXML<T>(object xmlobject)
        {
            XmlSerializer xsSubmit = new XmlSerializer(typeof(T));
            var subReq = xmlobject;

            using (var sww = new StringWriter())
            {
                using (XmlWriter writer = XmlWriter.Create(sww))
                {
                    xsSubmit.Serialize(writer, subReq);
                    return sww.ToString();
                }
            }
        }
    }
}
